@@ -29,6 +29,8 @@
 #include "egg/egg-libgcrypt.h"
 #include "egg/egg-secure-memory.h"
 
+#include "gcr/gcr-unlock-options-widget.h"
+
 #include <gcrypt.h>
 
 #include <glib/gi18n.h>
@@ -158,6 +160,12 @@ on_password_changed (GtkEditable *editable, gpointer user_data)
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (user_data), pwstrength);
 }
 
+static void
+on_auto_check_unlock_toggled (GtkToggleButton *check, GtkBuilder *builder)
+{
+	GtkWidget *area = GTK_WIDGET (gtk_builder_get_object (builder, "options_area"));
+	gtk_widget_set_sensitive (area, !gtk_toggle_button_get_active (check));
+}
 
 static void
 prepare_visibility (GtkBuilder *builder, GtkDialog *dialog)
@@ -304,42 +312,44 @@ prepare_security (GtkBuilder *builder, GtkDialog *dialog)
 	g_signal_connect (dialog, "window-state-event", G_CALLBACK (window_state_changed), NULL);
 }
 
-static void
-on_auto_check_unlock_toggled (GtkToggleButton *check, GtkBuilder *builder)
-{
-	GtkWidget *area;
-
-	area = GTK_WIDGET (gtk_builder_get_object (builder, "options_area"));
-	gtk_widget_set_sensitive (area, !gtk_toggle_button_get_active (check));
-}
-
-static void
-on_timeout_choices_toggled (GtkToggleButton *unused, GtkBuilder *builder)
-{
-	GtkWidget *spin, *after, *idle;
-
-	spin = GTK_WIDGET (gtk_builder_get_object (builder, "lock_minutes_spin"));
-	after = GTK_WIDGET (gtk_builder_get_object (builder, "lock_after_choice"));
-	idle = GTK_WIDGET (gtk_builder_get_object (builder, "lock_idle_choice"));
-	gtk_widget_set_sensitive (spin, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (after)) ||
-	                                gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(idle)));
-
-}
-
 static void
 prepare_lock (GtkBuilder *builder, GtkDialog *dialog)
 {
-	GtkWidget *check;
+	GtkWidget *unlock, *area;
+	gboolean unlock_auto;
+	GtkToggleButton *button;
+	gint unlock_idle, unlock_timeout;
 
-	check = GTK_WIDGET (gtk_builder_get_object (builder, "auto_unlock_check"));
-	g_signal_connect (check, "toggled", G_CALLBACK (on_auto_check_unlock_toggled), builder);
-	on_auto_check_unlock_toggled (GTK_TOGGLE_BUTTON (check), builder);
+	button = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "auto_unlock_check"));
+	g_signal_connect (button, "toggled", G_CALLBACK (on_auto_check_unlock_toggled), builder);
+	unlock_auto = g_key_file_get_boolean (input_data, "unlock-options", "unlock-auto", NULL);
+	gtk_toggle_button_set_active (button, unlock_auto);
+	on_auto_check_unlock_toggled (button, builder);
 
-	check = GTK_WIDGET (gtk_builder_get_object (builder, "lock_after_choice"));
-	g_signal_connect (check, "toggled", G_CALLBACK (on_timeout_choices_toggled), builder);
-	check = GTK_WIDGET (gtk_builder_get_object (builder, "lock_idle_choice"));
-	g_signal_connect (check, "toggled", G_CALLBACK (on_timeout_choices_toggled), builder);
-	on_timeout_choices_toggled (GTK_TOGGLE_BUTTON (check), builder);
+	unlock = gcr_unlock_options_widget_new ();
+	area = GTK_WIDGET (gtk_builder_get_object (builder, "options_area"));
+	g_object_set_data (G_OBJECT (dialog), "unlock-options-widget", unlock);
+	gtk_container_add (GTK_CONTAINER (area), unlock);
+	gtk_widget_show (unlock);
+
+	unlock_idle = g_key_file_get_integer (input_data, "unlock-options", "unlock-idle", NULL);
+	unlock_timeout = g_key_file_get_integer (input_data, "unlock-options", "unlock-timeout", NULL);
+
+	g_object_set (unlock,
+	              "unlock-idle", unlock_idle,
+	              "unlock-timeout", unlock_timeout,
+	              NULL);
+}
+
+static void
+prepare_details (GtkBuilder *builder, GtkDialog *dialog)
+{
+	GtkExpander *expander;
+	gboolean expanded;
+
+	expander = GTK_EXPANDER (gtk_builder_get_object (builder, "details_area"));
+	expanded = g_key_file_get_boolean (input_data, "details", "expanded", NULL);
+	gtk_expander_set_expanded (expander, expanded);
 }
 
 static GtkDialog*
@@ -365,6 +375,7 @@ prepare_dialog (GtkBuilder *builder)
 	prepare_passwords (builder, dialog);
 	prepare_security (builder, dialog);
 	prepare_lock (builder, dialog);
+	prepare_details (builder, dialog);
 
 	return dialog;
 }
@@ -553,12 +564,46 @@ gather_response (gint response)
 	g_key_file_set_value (output_data, "prompt", "response", value);
 }
 
+static void
+gather_unlock_options (GtkBuilder *builder, GtkDialog *dialog)
+{
+	gint unlock_timeout, unlock_idle;
+	GtkToggleButton *button;
+	GtkWidget *unlock;
+
+	button = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "auto_unlock_check"));
+	g_key_file_set_boolean (output_data, "unlock-options", "unlock-auto",
+	                        gtk_toggle_button_get_active (button));
+
+	unlock = g_object_get_data (G_OBJECT (dialog), "unlock-options-widget");
+	g_object_get (unlock,
+	              "unlock-timeout", &unlock_timeout,
+	              "unlock-idle", &unlock_idle,
+	              NULL);
+
+	g_key_file_set_integer (output_data, "unlock-options", "unlock-timeout", unlock_timeout);
+	g_key_file_set_integer (output_data, "unlock-options", "unlock-idle", unlock_idle);
+}
+
+static void
+gather_details (GtkBuilder *builder, GtkDialog *dialog)
+{
+	GtkExpander *expander;
+
+	expander = GTK_EXPANDER (gtk_builder_get_object (builder, "details_area"));
+	g_key_file_set_boolean (output_data, "details", "expanded",
+	                        gtk_expander_get_expanded (expander));
+}
+
+
 static void
 gather_dialog (GtkBuilder *builder, GtkDialog *dialog)
 {
 	gather_password (builder, "password");
 	gather_password (builder, "confirm");
 	gather_password (builder, "original");
+	gather_unlock_options (builder, dialog);
+	gather_details (builder, dialog);
 }
 
 static void
@@ -753,13 +798,15 @@ write_all_output (const gchar *data, gsize len)
 
 	while (len > 0) {
 		res = write (1, data, len);
-		if (res <= 0) {
+		if (res < 0) {
 			if (errno == EAGAIN || errno == EINTR)
 				continue;
 			if (errno != EPIPE)
 				g_warning ("couldn't write dialog response to output: %s",
 				           g_strerror (errno));
 			exit (1);
+		} else if (res == 0) {
+			g_warning ("couldn't write all dialog response to output");
 		} else  {
 			len -= res;
 			data += res;
