@@ -31,8 +31,12 @@
 #include "egg/egg-secure-memory.h"
 #include "egg/egg-spawn.h"
 
+#include "pkcs11/pkcs11i.h"
+
 #include <gcrypt.h>
 
+#include <sys/wait.h>
+
 #define DEBUG_PROMPT 1
 #define DEBUG_STDERR 0
 
@@ -78,6 +82,14 @@ G_DEFINE_TYPE (GkdPrompt, gkd_prompt, G_TYPE_OBJECT);
 /* Forward declaration*/
 static void display_async_prompt (GkdPrompt *);
 
+/* User choices we transfer over during a soft prompt reset */
+const struct { const gchar *section; const gchar *name; } SOFT_RESET[] = {
+	{ "unlock-options", "unlock-auto"},
+	{ "unlock-options", "unlock-idle"},
+	{ "unlock-options", "unlock-timeout"},
+	{ "details", "expanded" },
+};
+
 /* -----------------------------------------------------------------------------
  * INTERNAL
  */
@@ -514,7 +526,7 @@ static void
 gkd_prompt_init (GkdPrompt *self)
 {
 	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, GKD_TYPE_PROMPT, GkdPromptPrivate);
-	gkd_prompt_reset (self);
+	gkd_prompt_reset (self, TRUE);
 }
 
 static void
@@ -741,15 +753,33 @@ gkd_prompt_set_warning (GkdPrompt *self, const gchar *warning)
 }
 
 void
-gkd_prompt_reset (GkdPrompt *self)
+gkd_prompt_reset (GkdPrompt *self, gboolean hard)
 {
+	GKeyFile *input;
+	gchar *value;
+	gint i;
+
 	g_return_if_fail (GKD_IS_PROMPT (self));
 
 	kill_process (self);
 	self->pv->pid = 0;
 
+	input = g_key_file_new ();
+
+	/* If not a hard reset, copy over some user data */
+	if (!hard && self->pv->output) {
+		for (i = 0; i < G_N_ELEMENTS (SOFT_RESET); ++i) {
+			value = g_key_file_get_value (self->pv->output, SOFT_RESET[i].section,
+			                              SOFT_RESET[i].name, NULL);
+			if (value != NULL)
+				g_key_file_set_value (input, SOFT_RESET[i].section,
+				                      SOFT_RESET[i].name, value);
+			g_free (value);
+		}
+	}
+
 	clear_prompt_data (self);
-	self->pv->input = g_key_file_new ();
+	self->pv->input = input;
 }
 
 
@@ -811,6 +841,56 @@ gkd_prompt_get_transport_password (GkdPrompt *self, const gchar *password_type,
 	return TRUE;
 }
 
+void
+gkd_prompt_get_unlock_options (GkdPrompt *self, GP11Attributes *attrs)
+{
+	gboolean bval;
+	gint ival;
+
+	g_return_if_fail (GKD_IS_PROMPT (self));
+	g_return_if_fail (attrs);
+	g_return_if_fail (self->pv->output);
+
+	gp11_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
+
+	bval = g_key_file_get_boolean (self->pv->output, "unlock-options", "unlock-auto", NULL);
+	gp11_attributes_add_boolean (attrs, CKA_GNOME_TRANSIENT, !bval);
+
+	ival = g_key_file_get_integer (self->pv->output, "unlock-options", "unlock-idle", NULL);
+	gp11_attributes_add_ulong (attrs, CKA_G_DESTRUCT_IDLE, ival <= 0 ? 0 : ival);
+
+	ival = g_key_file_get_integer (self->pv->output, "unlock-options", "unlock-timeout", NULL);
+	gp11_attributes_add_ulong (attrs, CKA_G_DESTRUCT_AFTER, ival <= 0 ? 0 : ival);
+}
+
+void
+gkd_prompt_set_unlock_options (GkdPrompt *self, GP11Attributes *attrs)
+{
+	gboolean bval;
+	gulong uval;
+
+	g_return_if_fail (GKD_IS_PROMPT (self));
+	g_return_if_fail (attrs);
+	g_return_if_fail (self->pv->input);
+
+	if (gp11_attributes_find_boolean (attrs, CKA_GNOME_TRANSIENT, &bval))
+		g_key_file_set_boolean (self->pv->input, "unlock-options", "unlock-auto", !bval);
+
+	if (gp11_attributes_find_ulong (attrs, CKA_G_DESTRUCT_IDLE, &uval))
+		g_key_file_set_integer (self->pv->input, "unlock-options", "unlock-idle", (int)uval);
+
+	if (gp11_attributes_find_ulong (attrs, CKA_G_DESTRUCT_AFTER, &uval))
+		g_key_file_set_integer (self->pv->input, "unlock-options", "unlock-timeout", (int)uval);
+}
+
+gboolean
+gkd_prompt_get_unlock_auto (GkdPrompt *self)
+{
+	g_return_val_if_fail (GKD_IS_PROMPT (self), FALSE);
+	g_return_val_if_fail (self->pv->output, FALSE);
+	return g_key_file_get_boolean (self->pv->output, "unlock-options", "unlock-auto", NULL);
+}
+
 /* ----------------------------------------------------------------------------------
  * ATTENTION QUEUES
  */
