@@ -45,6 +45,22 @@ gck_attribute_get_bool (CK_ATTRIBUTE_PTR attr, gboolean *value)
 	return CKR_OK;
 }
 
+CK_RV
+gck_attribute_get_ulong (CK_ATTRIBUTE_PTR attr, CK_ULONG *value)
+{
+	CK_ULONG* ulong;
+
+	g_return_val_if_fail (attr, CKR_GENERAL_ERROR);
+	g_return_val_if_fail (value, CKR_GENERAL_ERROR);
+
+	if (attr->ulValueLen != sizeof (CK_ULONG) || attr->pValue == NULL)
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	ulong = attr->pValue;
+	*value = *ulong;
+	return CKR_OK;
+}
+
 #ifndef HAVE_TIMEGM
 static time_t
 timegm (struct tm *t)
@@ -110,6 +126,68 @@ gck_attribute_get_time (CK_ATTRIBUTE_PTR attr, glong *when)
 	return CKR_OK;
 }
 
+CK_RV
+gck_attribute_get_string (CK_ATTRIBUTE_PTR attr, gchar **value)
+{
+	g_return_val_if_fail (attr, CKR_GENERAL_ERROR);
+	g_return_val_if_fail (value, CKR_GENERAL_ERROR);
+
+	if (attr->ulValueLen == 0) {
+		*value = NULL;
+		return CKR_OK;
+	}
+
+	if (!attr->pValue)
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	if (!g_utf8_validate (attr->pValue, attr->ulValueLen, NULL))
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	*value = g_strndup (attr->pValue, attr->ulValueLen);
+	return CKR_OK;
+}
+
+CK_RV
+gck_attribute_get_mpi (CK_ATTRIBUTE_PTR attr, gcry_mpi_t *value)
+{
+	gcry_error_t gcry;
+
+	g_return_val_if_fail (attr, CKR_GENERAL_ERROR);
+	g_return_val_if_fail (value, CKR_GENERAL_ERROR);
+
+	gcry = gcry_mpi_scan (value, GCRYMPI_FMT_USG, attr->pValue, attr->ulValueLen, NULL);
+	if (gcry != 0)
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	return CKR_OK;
+}
+
+CK_RV
+gck_attribute_get_template (CK_ATTRIBUTE_PTR attr, GArray **template)
+{
+	CK_ULONG n_attrs;
+
+	g_return_val_if_fail (attr, CKR_GENERAL_ERROR);
+	g_return_val_if_fail (attr, CKR_GENERAL_ERROR);
+
+	/* Validate everything first */
+	if (attr->ulValueLen % sizeof (CK_ATTRIBUTE) != 0)
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	n_attrs = attr->ulValueLen / sizeof (CK_ATTRIBUTE);
+	if (n_attrs != 0 && !attr->pValue)
+		return CKR_ATTRIBUTE_VALUE_INVALID;
+
+	*template = gck_template_new (attr->pValue, n_attrs);
+	return CKR_OK;
+}
+
+CK_RV
+gck_attribute_set_empty (CK_ATTRIBUTE_PTR attr)
+{
+	return gck_attribute_set_data (attr, "", 0);
+}
+
 CK_RV
 gck_attribute_set_bool (CK_ATTRIBUTE_PTR attr, CK_BBOOL value)
 {
@@ -216,7 +294,7 @@ gck_attribute_set_mpi (CK_ATTRIBUTE_PTR attr, gcry_mpi_t mpi)
 	}
 	
 	if (len > attr->ulValueLen) {
-		attr->ulValueLen = len;
+		attr->ulValueLen = (CK_ULONG)-1;
 		return CKR_BUFFER_TOO_SMALL;
 	}
 
@@ -228,6 +306,49 @@ gck_attribute_set_mpi (CK_ATTRIBUTE_PTR attr, gcry_mpi_t mpi)
 	return CKR_OK;
 }
 
+CK_RV
+gck_attribute_set_template (CK_ATTRIBUTE_PTR attr, GArray *template)
+{
+	CK_ATTRIBUTE_PTR array;
+	CK_ATTRIBUTE_PTR at;
+	CK_RV rv;
+	gulong len;
+	gulong i;
+
+	g_assert (attr);
+	g_warn_if_fail ((attr->type & CKF_ARRAY_ATTRIBUTE) != 0);
+
+	len = sizeof (CK_ATTRIBUTE) * template->len;
+	if (!attr->pValue) {
+		attr->ulValueLen = len;
+		return CKR_OK;
+	} else if (len > attr->ulValueLen) {
+		attr->ulValueLen = (CK_ULONG)-1;
+		return CKR_BUFFER_TOO_SMALL;
+	}
+
+	attr->ulValueLen = len;
+	array = attr->pValue;
+	rv = CKR_OK;
+
+	/* Start working with individual elements */
+	for (i = 0; i < template->len; ++i) {
+		at = &g_array_index (template, CK_ATTRIBUTE, i);
+		array[i].type = at->type;
+		if (!array[i].pValue) {
+			array[i].ulValueLen = at->ulValueLen;
+		} else if (array[i].ulValueLen < at->ulValueLen) {
+			array[i].ulValueLen = (CK_ULONG)-1;
+			rv = CKR_BUFFER_TOO_SMALL;
+		} else {
+			memcpy(array[i].pValue, at->pValue, at->ulValueLen);
+			array[i].ulValueLen = at->ulValueLen;
+		}
+	}
+
+	return rv;
+}
+
 gboolean
 gck_attribute_equal (gconstpointer v1, gconstpointer v2)
 {
@@ -390,22 +511,105 @@ gck_attributes_find_ulong (CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs, CK_ATTRIBUT
 }
 
 gboolean
-gck_attributes_find_mpi (CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs, CK_ATTRIBUTE_TYPE type, gcry_mpi_t *value)
+gck_attributes_find_mpi (CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs,
+                         CK_ATTRIBUTE_TYPE type, gcry_mpi_t *value)
 {
 	CK_ATTRIBUTE_PTR attr;
-	gcry_error_t gcry;
-	
+
 	g_assert (attrs || !n_attrs);
-	
+
 	attr = gck_attributes_find (attrs, n_attrs, type);
 	if (attr == NULL)
 		return FALSE;
-	
-	if (value != NULL) {
-		gcry = gcry_mpi_scan (value, GCRYMPI_FMT_USG, attr->pValue, attr->ulValueLen, NULL);
-		if (gcry != 0)
-			return FALSE;
+
+	return gck_attribute_get_mpi (attr, value) == CKR_OK;
+}
+
+gboolean
+gck_attributes_find_string (CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs,
+                            CK_ATTRIBUTE_TYPE type, gchar **value)
+{
+	CK_ATTRIBUTE_PTR attr;
+
+	g_return_val_if_fail (attrs || !n_attrs, FALSE);
+
+	attr = gck_attributes_find (attrs, n_attrs, type);
+	if (attr == NULL)
+		return FALSE;
+
+	return gck_attribute_get_string (attr, value) == CKR_OK;
+}
+
+GArray*
+gck_template_new (CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs)
+{
+	GArray *template = g_array_new (FALSE, FALSE, sizeof (CK_ATTRIBUTE));
+	CK_ATTRIBUTE_PTR pat;
+	gulong i;
+
+	g_return_val_if_fail (attrs || !n_attrs, NULL);
+
+	g_array_append_vals (template, attrs, n_attrs);
+	for (i = 0; i < n_attrs; ++i) {
+		pat = &g_array_index (template, CK_ATTRIBUTE, i);
+		if (pat->pValue)
+			pat->pValue = g_memdup (pat->pValue, pat->ulValueLen);
 	}
-	
-	return TRUE;
+
+	return template;
+}
+
+void
+gck_template_set (GArray *template, CK_ATTRIBUTE_PTR attr)
+{
+	CK_ATTRIBUTE_PTR pat;
+	CK_ATTRIBUTE at;
+	guint i;
+
+	g_return_if_fail (template);
+	g_return_if_fail (attr);
+
+	pat = gck_attributes_find ((CK_ATTRIBUTE_PTR)template->data, template->len, attr->type);
+
+	/* Remove any previous value */
+	for (i = 0; i < template->len; ++i) {
+		if (g_array_index (template, CK_ATTRIBUTE, i).type == attr->type) {
+			g_free (g_array_index (template, CK_ATTRIBUTE, i).pValue);
+			g_array_remove_index_fast (template, i);
+			break;
+		}
+	}
+
+	/* Add a new attribute */
+	memcpy (&at, attr, sizeof (at));
+	if (at.pValue)
+		at.pValue = g_memdup (at.pValue, at.ulValueLen);
+	g_array_append_vals (template, &at, 1);
+}
+
+void
+gck_template_free (GArray *template)
+{
+	guint i;
+
+	if (!template)
+		return;
+
+	for (i = 0; i < template->len; ++i)
+		g_free (g_array_index (template, CK_ATTRIBUTE, i).pValue);
+	g_array_free (template, TRUE);
+}
+
+gboolean
+gck_template_find_boolean (GArray *template, CK_ATTRIBUTE_TYPE type, gboolean *value)
+{
+	g_return_val_if_fail (template, FALSE);
+	return gck_attributes_find_boolean ((CK_ATTRIBUTE_PTR)template->data, template->len, type, value);
+}
+
+gboolean
+gck_template_find_ulong (GArray *template, CK_ATTRIBUTE_TYPE type, gulong *value)
+{
+	g_return_val_if_fail (template, FALSE);
+	return gck_attributes_find_ulong ((CK_ATTRIBUTE_PTR)template->data, template->len, type, value);
 }
