@@ -25,7 +25,9 @@
 #include "gck-ssh-private-key.h"
 
 #include "gck/gck-attributes.h"
+#include "gck/gck-credential.h"
 #include "gck/gck-manager.h"
+#include "gck/gck-module.h"
 #include "gck/gck-object.h"
 #include "gck/gck-sexp.h"
 #include "gck/gck-util.h"
@@ -41,25 +43,25 @@ enum {
 };
 
 struct _GckSshPrivateKey {
-	GckPrivateKey parent;
-	
+	GckPrivateXsaKey parent;
+
 	GckSshPublicKey *pubkey;
 	gchar *label;
 	guchar *private_data;
 	gsize n_private_data;
 	
-	GckSexp *private_sexp;
 	gboolean is_encrypted;
 };
 
-G_DEFINE_TYPE (GckSshPrivateKey, gck_ssh_private_key, GCK_TYPE_PRIVATE_KEY);
+G_DEFINE_TYPE (GckSshPrivateKey, gck_ssh_private_key, GCK_TYPE_PRIVATE_XSA_KEY);
 
 /* -----------------------------------------------------------------------------
  * INTERNAL 
  */
 
 static CK_RV
-unlock_private_key (GckSshPrivateKey *self, const gchar *password, gssize n_password)
+unlock_private_key (GckSshPrivateKey *self, const gchar *password,
+                    gssize n_password, GckSexp **result)
 {
 	GckDataResult res;
 	gcry_sexp_t sexp;
@@ -89,11 +91,10 @@ unlock_private_key (GckSshPrivateKey *self, const gchar *password, gssize n_pass
 
 	if (!password || !password[0])
 		self->is_encrypted = FALSE;
-
-	wrapper = gck_sexp_new (sexp);
-	gck_private_key_store_private (GCK_PRIVATE_KEY (self), wrapper, self->is_encrypted ? 1 : G_MAXUINT);
-	gck_sexp_unref (wrapper);
 	
+	wrapper = gck_sexp_new (sexp);
+	*result = wrapper;
+
 	return CKR_OK;
 }
 
@@ -107,8 +108,8 @@ realize_and_take_data (GckSshPrivateKey *self, gcry_sexp_t sexp, gchar *comment,
 
 	/* The base public key gets setup. */
 	wrapper = gck_sexp_new (sexp);
-	gck_key_set_base_sexp (GCK_KEY (self), wrapper);
-	gck_key_set_base_sexp (GCK_KEY (self->pubkey), wrapper);
+	gck_sexp_key_set_base (GCK_SEXP_KEY (self), wrapper);
+	gck_sexp_key_set_base (GCK_SEXP_KEY (self->pubkey), wrapper);
 	gck_sexp_unref (wrapper);
 
 	/* Own the comment */
@@ -121,13 +122,13 @@ realize_and_take_data (GckSshPrivateKey *self, gcry_sexp_t sexp, gchar *comment,
 	self->private_data = private_data;
 	self->n_private_data = n_private_data;
 
-	/* Force parsing next time required */
-	gck_private_key_store_private (GCK_PRIVATE_KEY (self), NULL, 0);
-	
 	/* Try to parse the private data, and note if it's not actually encrypted */
 	self->is_encrypted = TRUE;
-	if (unlock_private_key (self, "", 0) == CKR_OK) 
+	if (unlock_private_key (self, "", 0, &wrapper) == CKR_OK) {
 		self->is_encrypted = FALSE;
+		gck_private_xsa_key_set_unlocked_private (GCK_PRIVATE_XSA_KEY (self), wrapper);
+		gck_sexp_unref (wrapper);
+	}
 }
 
 /* -----------------------------------------------------------------------------
@@ -135,7 +136,7 @@ realize_and_take_data (GckSshPrivateKey *self, gcry_sexp_t sexp, gchar *comment,
  */
 
 static CK_RV
-gck_ssh_private_key_get_attribute (GckObject *base, CK_ATTRIBUTE_PTR attr)
+gck_ssh_private_key_get_attribute (GckObject *base, GckSession *session, CK_ATTRIBUTE_PTR attr)
 {
 	GckSshPrivateKey *self = GCK_SSH_PRIVATE_KEY (base);
 	gchar *digest;
@@ -155,26 +156,51 @@ gck_ssh_private_key_get_attribute (GckObject *base, CK_ATTRIBUTE_PTR attr)
 		return rv;
 	}
 	
-	return GCK_OBJECT_CLASS (gck_ssh_private_key_parent_class)->get_attribute (base, attr);
+	return GCK_OBJECT_CLASS (gck_ssh_private_key_parent_class)->get_attribute (base, session, attr);
 }
 
 static CK_RV
-gck_ssh_private_key_unlock (GckObject *base, CK_UTF8CHAR_PTR pin, CK_ULONG n_pin)
+gck_ssh_private_key_unlock (GckObject *base, GckCredential *cred)
 {
 	GckSshPrivateKey *self = GCK_SSH_PRIVATE_KEY (base);
-	return unlock_private_key (self, (const gchar*)pin, n_pin);
+	const gchar *password;
+	GckSexp *wrapper;
+	gsize n_password;
+	CK_RV rv;
+
+	if (!self->is_encrypted)
+		return CKR_OK;
+
+	password = gck_credential_get_password (cred, &n_password);
+	rv = unlock_private_key (self, password, n_password, &wrapper);
+
+	if (rv == CKR_OK) {
+		gck_private_xsa_key_set_locked_private (GCK_PRIVATE_XSA_KEY (self), cred, wrapper);
+		gck_sexp_unref (wrapper);
+	}
+
+	return rv;
+}
+
+static void
+gck_ssh_private_key_expose (GckObject *base, gboolean expose)
+{
+	GCK_OBJECT_CLASS (gck_ssh_private_key_parent_class)->expose_object (base, expose);
+	gck_object_expose (GCK_OBJECT (GCK_SSH_PRIVATE_KEY (base)->pubkey), expose);
 }
 
 static GObject* 
 gck_ssh_private_key_constructor (GType type, guint n_props, GObjectConstructParam *props) 
 {
 	GckSshPrivateKey *self = GCK_SSH_PRIVATE_KEY (G_OBJECT_CLASS (gck_ssh_private_key_parent_class)->constructor(type, n_props, props));
+	GckObject *object;
 	gchar *unique;
 	
 	g_return_val_if_fail (self, NULL);	
 
-	unique = g_strdup_printf ("%s.pub", gck_object_get_unique (GCK_OBJECT (self)));
-	self->pubkey = gck_ssh_public_key_new (unique);
+	object = GCK_OBJECT (self);
+	unique = g_strdup_printf ("%s.pub", gck_object_get_unique (object));
+	self->pubkey = gck_ssh_public_key_new (gck_object_get_module (object), unique);
 	g_free (unique);
 	
 	return G_OBJECT (self);
@@ -263,6 +289,7 @@ gck_ssh_private_key_class_init (GckSshPrivateKeyClass *klass)
 	
 	gck_class->get_attribute = gck_ssh_private_key_get_attribute;
 	gck_class->unlock = gck_ssh_private_key_unlock;
+	gck_class->expose_object = gck_ssh_private_key_expose;
 	
 	g_object_class_install_property (gobject_class, PROP_LABEL,
 	           g_param_spec_string ("label", "Label", "Object Label", 
@@ -278,9 +305,10 @@ gck_ssh_private_key_class_init (GckSshPrivateKeyClass *klass)
  */
 
 GckSshPrivateKey*
-gck_ssh_private_key_new (const gchar *unique)
+gck_ssh_private_key_new (GckModule *module, const gchar *unique)
 {
-	return g_object_new (GCK_TYPE_SSH_PRIVATE_KEY, "unique", unique, NULL);
+	return g_object_new (GCK_TYPE_SSH_PRIVATE_KEY, "unique", unique, 
+	                     "module", module, "manager", gck_module_get_manager (module), NULL);
 }
 
 gboolean
@@ -304,8 +332,10 @@ gck_ssh_private_key_parse (GckSshPrivateKey *self, const gchar *public_path,
 	/* Parse it */
 	res = gck_ssh_openssh_parse_public_key (public_data, n_public_data, &sexp, &comment);
 	g_free (public_data);
-	
-	if (res != GCK_DATA_SUCCESS) {
+
+	if (res == GCK_DATA_UNRECOGNIZED) {
+		return FALSE;
+	} else if (res != GCK_DATA_SUCCESS) {
 		g_set_error_literal (error, GCK_DATA_ERROR, res, _("Couldn't parse public SSH key"));
 		return FALSE;
 	}
