@@ -23,6 +23,8 @@
 
 /* This file is included into the main .c file for each gtest unit-test program */
 
+#include "config.h"
+
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <stdio.h>
@@ -35,6 +37,15 @@
 
 #include "egg/egg-secure-memory.h"
 
+#include "pkcs11/pkcs11.h"
+
+#ifdef WITH_P11_TESTS
+#include <p11-tests.h>
+#endif
+
+/* Forward declaration */
+void test_p11_module (CK_FUNCTION_LIST_PTR module, const gchar *config);
+
 static GStaticMutex memory_mutex = G_STATIC_MUTEX_INIT;
 static const gchar *test_path = NULL;
 
@@ -88,22 +99,129 @@ test_mainloop_get (void)
 	return mainloop;
 }
 
+const gchar*
+test_scratch_directory (void)
+{
+	return test_path;
+}
+
 gchar*
-test_build_filename (const gchar *basename)
+test_scratch_filename (const gchar *basename)
 {
 	return g_build_filename (test_path, basename, NULL);
 }
 
-guchar* 
-test_read_testdata (const gchar *basename, gsize *n_result)
+gchar*
+test_data_filename (const gchar *basename)
 {
-	gchar *file = g_build_filename ("test-data", basename, NULL);
+	return g_build_filename (test_data_directory (), basename, NULL);
+}
+
+const gchar*
+test_data_directory (void)
+{
+	const gchar *dir;
+	gchar *cur, *env;
+
+	dir = g_getenv ("TEST_DATA");
+	if (dir == NULL)
+		dir = "./test-data";
+	if (!g_path_is_absolute (dir)) {
+		cur = g_get_current_dir ();
+		if (strncmp (dir, "./", 2) == 0)
+			dir += 2;
+		env = g_build_filename (cur, dir, NULL);
+		g_free (cur);
+		g_setenv ("TEST_DATA", env, TRUE);
+		g_free (env);
+		dir = g_getenv ("TEST_DATA");
+	}
+
+	return dir;
+}
+
+guchar* 
+test_data_read (const gchar *basename, gsize *n_result)
+{
+	GError *error = NULL;
 	gchar *result;
-	if (!g_file_get_contents (file, &result, n_result, NULL))
+	gchar *file;
+
+	file = test_data_filename (basename);
+	if (!g_file_get_contents (file, &result, n_result, &error)) {
+		g_warning ("could not read test data file: %s: %s", file,
+		           error && error->message ? error->message : "");
 		g_assert_not_reached ();
+	}
+
+	g_free (file);
 	return (guchar*)result;
 }
 
+#if WITH_P11_TESTS
+
+static void
+on_p11_tests_log (int level, const char *section, const char *message)
+{
+	if (level == P11_TESTS_NONE) {
+		g_message ("%s", message);
+	} else if (level != P11_TESTS_FAIL) {
+		g_message ("%s: %s", section, message);
+	} else {
+		g_print ("/%s/%s: FAIL: %s\n", test_external_name (), section, message);
+		test_external_fail ();
+	}
+}
+
+void
+test_p11_module (CK_FUNCTION_LIST_PTR module, const gchar *config)
+{
+	p11_tests_set_log_func (on_p11_tests_log);
+	p11_tests_set_unexpected (1);
+	p11_tests_set_verbose (0);
+	p11_tests_set_write_session (1);
+	if (config)
+		p11_tests_load_config (config);
+	p11_tests_perform (module);
+}
+
+#else /* !WITH_P11_TESTS */
+
+void
+test_p11_module (CK_FUNCTION_LIST_PTR module, const gchar *config)
+{
+	g_message ("p11-tests support not built in");
+}
+
+#endif /* !WITH_P11_TESTS */
+
+static const gchar *external_name = NULL;
+static gint external_fails = 0;
+
+void
+test_external_run (const gchar *name, TestExternalFunc func)
+{
+	external_fails = 0;
+	external_name = name;
+	func ();
+	if (external_fails) {
+		g_printerr ("/%s: FAIL: %d failures", name, external_fails);
+		abort();
+	}
+}
+
+const gchar*
+test_external_name (void)
+{
+	return external_name;
+}
+
+void
+test_external_fail (void)
+{
+	++external_fails;
+}
+
 static void 
 chdir_base_dir (char* argv0)
 {
@@ -121,7 +239,6 @@ chdir_base_dir (char* argv0)
 			           g_strerror (errno));
 	}
 
-	g_free (base);
 	g_free (dir);
 }
 
@@ -133,9 +250,7 @@ main (int argc, char* argv[])
 	g_thread_init (NULL);
 
 	test_path = getenv ("GNOME_KEYRING_TEST_PATH");
-	if (test_path) {
-		setenv ("GNOME_KEYRING_OUTSIDE_TEST", "TRUE", 1);
-	} else {
+	if (!test_path) {
 		test_path = "/tmp/test-gnome-keyring";
 		setenv ("GNOME_KEYRING_TEST_PATH", test_path, 1);
 		g_mkdir_with_parents (test_path, 0777);
@@ -150,6 +265,6 @@ main (int argc, char* argv[])
 	fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
 	g_log_set_always_fatal (fatal_mask);
 
-	initialize_tests ();
-	return g_test_run ();
-} 
+	/* Must have been defined by the test including this file */
+	return run();
+}
